
import 'dart:async';

import 'package:chobi_admob_flutter/chobi_ad_state.dart';
import 'package:chobi_admob_flutter/chobi_admob_flutter_platform_interface.dart';
import 'package:flutter/cupertino.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class ChobiAdWidget extends ConsumerStatefulWidget {

  static bool _isAdMobInitialized = false;
  static bool get isAdMobInitialized => _isAdMobInitialized;

  static Future<void> resetConsentState() async {
    await ConsentInformation.instance.reset();
  }

  final FutureOr<Ad?> Function(ChobiAdState adState) adBuilder;
  final StateProvider<ChobiAdState>? adStateProvider;
  final StateProvider<Ad?>? adProvider;
  final List<String>? testDeviceIds;
  final List<String>? umpTestDeviceIds;
  final bool isDebugMode;

  final void Function(ConsentStatus status)? onConsentCheckFinished;
  final void Function(FormError err)? onConsentError;

  const ChobiAdWidget({
    super.key,
    required this.adBuilder,
    this.adStateProvider,
    this.adProvider,
    this.testDeviceIds,
    this.umpTestDeviceIds,
    this.isDebugMode = true,
    this.onConsentCheckFinished,
    this.onConsentError,
  });


  @override
  ConsumerState<ChobiAdWidget> createState() => _ChobiAdWidgetState();
}

class _ChobiAdWidgetState extends ConsumerState<ChobiAdWidget> with SingleTickerProviderStateMixin, AutomaticKeepAliveClientMixin {

  late final StateProvider<ChobiAdState> _stateProvider;
  late final StateProvider<Ad?> _adProvider;


  void _updateState(WidgetRef ref, {
    bool? isAdsAvailable,
    bool? appTrackingConfirmed,
  }) {
    final old = ref.read(_stateProvider);
    ref.read(_stateProvider.notifier).state = ChobiAdState(
      isAdsAvailable: isAdsAvailable ?? old.isAdsAvailable,
      appTrackingConfirmed: appTrackingConfirmed ?? old.appTrackingConfirmed,
    );
  }

  @override
  void initState() {
    super.initState();

    _stateProvider = widget.adStateProvider ?? ChobiAdState.createDefaultProvider();
    _adProvider = widget.adProvider ?? StateProvider((ref) => null);

    WidgetsBinding.instance.addPostFrameCallback((_) async {
      await ChobiAdmobFlutterPlatform.instance.confirmAppTrackingTransparency();

      _updateState(ref, appTrackingConfirmed: true);


    });
  }

  ConsentRequestParameters _createParams() {
    final cSetting = ConsentDebugSettings();
    cSetting.testIdentifiers = widget.umpTestDeviceIds ?? widget.testDeviceIds;
    if (widget.isDebugMode) {
      cSetting.debugGeography = DebugGeography.debugGeographyEea;
    }

    return ConsentRequestParameters(consentDebugSettings: cSetting);
  }

  void _requestConsentInfoUpdate() {
    ConsentInformation.instance.requestConsentInfoUpdate(
      _createParams(),
          () async {
            final isFormAvailable = await ConsentInformation.instance.isConsentFormAvailable();
            if (isFormAvailable) {
              _showConsentForm();
            } else {
              _updateState(ref, isAdsAvailable: true);
            }
          },
          (error) {
            widget.onConsentError?.call(error);
          },
    );
  }

  void _showConsentForm() {
    ConsentForm.loadConsentForm(
          (consentForm) async {
            final status = await ConsentInformation.instance.getConsentStatus();
            if (status == ConsentStatus.required) {
              consentForm.show((formError) {
                if (formError != null) {
                  widget.onConsentError?.call(formError);
                }
                _showConsentForm();
              });
            } else {
              _updateState(ref, isAdsAvailable: true);
            }
          },
          (formError) => widget.onConsentError?.call(formError),
    );
  }



  Future<void> _initAdMob() async {
    if (ChobiAdWidget._isAdMobInitialized) {
      return;
    }

    ChobiAdWidget._isAdMobInitialized = true;

    await MobileAds.instance.initialize();
  }


  @override
  bool get wantKeepAlive => true;

}
